@@ -2,7 +2,7 @@
 # FlashGBX
 # Author: Lesserkuma (github.com/lesserkuma)
 
-import hashlib, re, string, struct
+import hashlib, re, string, struct, os, json, copy
 from . import Util
 
 try:
@@ -14,6 +14,7 @@ except:
 class RomFileDMG:
 	ROMFILE_PATH = None
 	ROMFILE = bytearray()
+	DATA = None
 	
 	def __init__(self, file=None):
 		if isinstance(file, str):
@@ -105,6 +106,12 @@ class RomFileDMG:
 		game_title = re.sub(r"(\x00+)$", "", game_title)
 		game_title = re.sub(r"((_)_+|(\x00)\x00+|(\s)\s+)", "\\2\\3\\4", game_title).replace("\x00", "_")
 		game_title = ''.join(filter(lambda x: x in set(string.printable), game_title))
+		data["game_code"] = ""
+		if data["cgb"] in (0x80, 0xC0):
+			if len(data["game_title_raw"].rstrip("\x00")) == 15:
+				if data["game_title_raw"][-4:][0] in ("A", "B", "H", "K", "V") and data["game_title_raw"][-4:][3] in ("A", "B", "D", "E", "F", "I", "J", "K", "P", "S", "U", "X", "Y"):
+					data["game_code"] = game_title[-4:]
+					game_title = game_title[:-4].rstrip("_")
 		data["game_title"] = game_title
 
 		data["maker_code"] = format(int(buffer[0x14B]), "02X")
@@ -124,6 +131,7 @@ class RomFileDMG:
 			data["ram_size"] = "?"
 			if buffer[0x149]  < len(Util.DMG_Header_RAM_Sizes):
 				data["ram_size"] = Util.DMG_Header_RAM_Sizes[buffer[0x149]]
+		data["header_sha1"] = hashlib.sha1(buffer[0x0:0x180]).hexdigest()
 		data["version"] = int(buffer[0x14C])
 		data["header_checksum"] = int(buffer[0x14D])
 		data["header_checksum_calc"] = self.CalcChecksumHeader()
@@ -131,22 +139,48 @@ class RomFileDMG:
 		data["rom_checksum"] = int(256 * buffer[0x14E] + buffer[0x14F])
 		data["rom_checksum_calc"] = self.CalcChecksumGlobal()
 		data["rom_checksum_correct"] = data["rom_checksum"] == data["rom_checksum_calc"]
-
+		
+		data["unchanged"] = copy.copy(data)
 		if not unchanged:
+			# MBC2
+			if data["mapper_raw"] == 0x06:
+				data["ram_size_raw"] = 0x100
+
+			# MBC30
+			if data["mapper_raw"] == 0x10 and data["ram_size_raw"] == 0x05:
+				data["mapper_raw"] += 0x100
+
+			# MBC6
+			if data["mapper_raw"] == 0x20:
+				data["ram_size_raw"] = 0x104
+
+			# MBC7
+			if data["mapper_raw"] == 0x22 and data["game_title"] in ("KORO2 KIRBY", "KIRBY TNT"):
+				data["ram_size_raw"] = 0x101
+			elif data["mapper_raw"] == 0x22 and data["game_title"] == "CMASTER":
+				data["ram_size_raw"] = 0x102
+
+			# TAMA5
+			if data["mapper_raw"] == 0xFD:
+				data["ram_size_raw"] = 0x103
+
 			# MBC1M
 			if data["mapper_raw"] == 0x03 and data["game_title"] == "MOMOCOL" and data["header_checksum"] == 0x28 or \
 			data["mapper_raw"] == 0x01 and data["game_title"] == "BOMCOL" and data["header_checksum"] == 0x86 or \
+			data["mapper_raw"] == 0x01 and data["game_title"] == "BOMSEL" and data["header_checksum"] == 0x9C or \
 			data["mapper_raw"] == 0x01 and data["game_title"] == "GENCOL" and data["header_checksum"] == 0x8A or \
 			data["mapper_raw"] == 0x01 and data["game_title"] == "SUPERCHINESE 123" and data["header_checksum"] == 0xE4 or \
 			data["mapper_raw"] == 0x01 and data["game_title"] == "MORTALKOMBATI&II" and data["header_checksum"] == 0xB9 or \
 			data["mapper_raw"] == 0x01 and data["game_title"] == "MORTALKOMBAT DUO" and data["header_checksum"] == 0xA7:
 				data["mapper_raw"] += 0x100
 
-			# GB Memory
+			# GB-Memory (DMG-MMSA-JPN)
 			if data["mapper_raw"] == 0x19 and data["game_title"] == "NP M-MENU MENU" and data["header_checksum"] == 0xD3:
+				data["rom_size_raw"] = 0x05
 				data["ram_size_raw"] = 0x04
 				data["mapper_raw"] = 0x105
 			elif data["mapper_raw"] == 0x01 and data["game_title"] == "DMG MULTI MENU " and data["header_checksum"] == 0x36:
+				data["rom_size_raw"] = 0x05
 				data["ram_size_raw"] = 0x04
 				data["mapper_raw"] = 0x105
 			
@@ -217,11 +251,13 @@ class RomFileDMG:
 					pass
 			
 			# Unlicensed Datel Orbit V2 Mapper (older firmware)
-			elif hashlib.sha1(buffer[0x101:0x140]).digest() == bytearray([ 0xC1, 0xF4, 0x15, 0x4A, 0xEF, 0xCC, 0x5B, 0xE7, 0xEC, 0x83, 0xA8, 0xBB, 0x7B, 0xC0, 0x95, 0x83, 0x35, 0xEC, 0x9A, 0xF2 ]):
+			elif (
+				hashlib.sha1(buffer[0x101:0x140]).digest() == bytearray([ 0xC1, 0xF4, 0x15, 0x4A, 0xEF, 0xCC, 0x5B, 0xE7, 0xEC, 0x83, 0xA8, 0xBB, 0x7B, 0xC0, 0x95, 0x83, 0x35, 0xEC, 0x9A, 0xF2 ]) or \
+				hashlib.sha1(buffer[0x101:0x140]).digest() == bytearray([ 0xC9, 0x50, 0x65, 0xCB, 0x31, 0x96, 0x26, 0x6C, 0x32, 0x58, 0xAB, 0x07, 0xA1, 0x9E, 0x0C, 0x10, 0xA6, 0xED, 0xCC, 0x67 ])
+			):
 				data["rom_size_raw"] = 0x02
 				data["ram_size_raw"] = 0
 				data["mapper_raw"] = 0x205
-				data["cgb"] = 0x80
 				try:
 					game_title = bytearray(buffer[0x134:0x140]).decode("ascii", "replace").replace("\xFF", "")
 					game_title = re.sub(r"(\x00+)$", "", game_title)
@@ -230,7 +266,21 @@ class RomFileDMG:
 					data["game_title"] = game_title
 				except:
 					pass
-
+			
+			# Unlicensed Datel Mega Memory Card
+			# elif hashlib.sha1(buffer[0x104:0x150]).digest() == bytearray([ 0x05, 0xBE, 0x44, 0xBA, 0xE8, 0x0A, 0x59, 0x76, 0x34, 0x1B, 0x01, 0xDF, 0x92, 0xDD, 0xAB, 0x8C, 0x7A, 0x2A, 0x8F, 0x4E ]):
+			# 	data["rom_size_raw"] = 0x00
+			# 	data["ram_size_raw"] = 0x206
+			# 	data["mapper_raw"] = 0x206
+			# 	try:
+			# 		game_title = bytearray(buffer[0:0x10]).decode("ascii", "replace").replace("\xFF", "")
+			# 		game_title = re.sub(r"(\x00+)$", "", game_title)
+			# 		game_title = re.sub(r"((_)_+|(\x00)\x00+|(\s)\s+)", "\\2\\3\\4", game_title).replace("\x00", "")
+			# 		game_title = ''.join(filter(lambda x: x in set(string.printable), game_title))
+			# 		data["game_title"] = game_title
+			# 	except:
+			# 		pass
+			
 			# Unlicensed Sachen MMC1/MMC2
 			elif len(buffer) >= 0x280:
 				sachen_version = 0
@@ -386,4 +436,21 @@ class RomFileDMG:
 		elif data["logo_correct"]:
 			print("{:s}WARNING: Unknown memory bank controller type 0x{:02X}{:s}".format(Util.ANSI.YELLOW, data["mapper_raw"], Util.ANSI.RESET))
 
+		self.DATA = data
+		data["db"] = self.GetDatabaseEntry()
+		if data["db"] is not None and data["game_code"] == "" and data["db"]["gc"] != "":
+			data["game_code"] = data["db"]["gc"][4:]
 		return data
+
+	def GetDatabaseEntry(self):
+		data = self.DATA
+		db_entry = None
+		if os.path.exists("{0:s}/db_DMG.json".format(Util.CONFIG_PATH)):
+			with open("{0:s}/db_DMG.json".format(Util.CONFIG_PATH), encoding="UTF-8") as f:
+				db = f.read()
+				db = json.loads(db)
+				if data["header_sha1"] in db.keys():
+					db_entry = db[data["header_sha1"]]
+		else:
+			print("FAIL: Database for Game Boy titles not found at {0:s}/db_DMG.json".format(Util.CONFIG_PATH))
+		return db_entry
